@@ -1,5 +1,5 @@
 /*******************************************************************************
- Copyright (c) 2022, Intel Corporation
+ Copyright (c) 2022-2023, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -48,6 +48,7 @@
 #include "include/aesni_emu.h"
 #include "include/error.h"
 
+#include "include/arch_sse_type2.h" /* SHA-NI */
 #include "include/arch_avx_type1.h" /* AESNI */
 #include "include/arch_avx2_type1.h" /* MD5 */
 #include "include/arch_avx512_type1.h"
@@ -79,6 +80,7 @@
 #define SUBMIT_CIPHER_BURST_NOCHECK submit_cipher_burst_nocheck_avx512_t2
 #define SUBMIT_HASH_BURST submit_hash_burst_avx512_t2
 #define SUBMIT_HASH_BURST_NOCHECK submit_hash_burst_nocheck_avx512_t2
+#define SET_SUITE_ID_FN   set_suite_id_avx512_t2
 
 /* Hash */
 #define SUBMIT_JOB_HASH    SUBMIT_JOB_HASH_AVX512_T2
@@ -118,10 +120,13 @@
 #define AES_CBC_DEC_256       aes_cbc_dec_256_vaes_avx512
 
 /* AES-CBCS */
-#define SUBMIT_JOB_AES128_CBCS_1_9_ENC submit_job_aes128_cbcs_1_9_enc_vaes_avx512
-#define FLUSH_JOB_AES128_CBCS_1_9_ENC  flush_job_aes128_cbcs_1_9_enc_vaes_avx512
-#define SUBMIT_JOB_AES128_CBCS_1_9_DEC submit_job_aes128_cbcs_1_9_dec_vaes_avx512
-#define AES_CBCS_1_9_DEC_128           aes_cbcs_1_9_dec_128_vaes_avx512
+#define SUBMIT_JOB_AES128_CBCS_1_9_ENC \
+        submit_job_aes128_cbcs_1_9_enc_vaes_avx512
+#define FLUSH_JOB_AES128_CBCS_1_9_ENC  \
+        flush_job_aes128_cbcs_1_9_enc_vaes_avx512
+#define SUBMIT_JOB_AES128_CBCS_1_9_DEC \
+        submit_job_aes128_cbcs_1_9_dec_vaes_avx512
+#define AES_CBCS_1_9_DEC_128 aes_cbcs_1_9_dec_128_vaes_avx512
 
 /* AES-ECB */
 #define SUBMIT_JOB_AES_ECB_128_ENC submit_job_aes_ecb_128_enc_avx512
@@ -339,12 +344,18 @@ submit_job_docsis256_sec_crc_dec_vaes_avx512(MB_MGR_DOCSIS_AES_OOO *state,
         return job;
 }
 
-#define SUBMIT_JOB_DOCSIS128_SEC_CRC_ENC submit_job_aes_docsis128_enc_crc32_vaes_avx512
-#define SUBMIT_JOB_DOCSIS256_SEC_CRC_ENC submit_job_aes_docsis256_enc_crc32_vaes_avx512
-#define FLUSH_JOB_DOCSIS128_SEC_CRC_ENC flush_job_aes_docsis128_enc_crc32_vaes_avx512
-#define FLUSH_JOB_DOCSIS256_SEC_CRC_ENC flush_job_aes_docsis256_enc_crc32_vaes_avx512
-#define SUBMIT_JOB_DOCSIS128_SEC_CRC_DEC submit_job_docsis128_sec_crc_dec_vaes_avx512
-#define SUBMIT_JOB_DOCSIS256_SEC_CRC_DEC submit_job_docsis256_sec_crc_dec_vaes_avx512
+#define SUBMIT_JOB_DOCSIS128_SEC_CRC_ENC \
+        submit_job_aes_docsis128_enc_crc32_vaes_avx512
+#define SUBMIT_JOB_DOCSIS256_SEC_CRC_ENC \
+        submit_job_aes_docsis256_enc_crc32_vaes_avx512
+#define FLUSH_JOB_DOCSIS128_SEC_CRC_ENC \
+        flush_job_aes_docsis128_enc_crc32_vaes_avx512
+#define FLUSH_JOB_DOCSIS256_SEC_CRC_ENC \
+        flush_job_aes_docsis256_enc_crc32_vaes_avx512
+#define SUBMIT_JOB_DOCSIS128_SEC_CRC_DEC \
+        submit_job_docsis128_sec_crc_dec_vaes_avx512
+#define SUBMIT_JOB_DOCSIS256_SEC_CRC_DEC \
+        submit_job_docsis256_sec_crc_dec_vaes_avx512
 
 /* ====================================================================== */
 
@@ -470,6 +481,7 @@ init_mb_mgr_avx512_t2_internal(IMB_MGR *state, const int reset_mgrs)
         state->submit_cipher_burst_nocheck = SUBMIT_CIPHER_BURST_NOCHECK;
         state->submit_hash_burst   = SUBMIT_HASH_BURST;
         state->submit_hash_burst_nocheck = SUBMIT_HASH_BURST_NOCHECK;
+        state->set_suite_id        = SET_SUITE_ID_FN;
 
         state->keyexp_128          = aes_keyexp_128_avx512;
         state->keyexp_192          = aes_keyexp_192_avx512;
@@ -482,12 +494,12 @@ init_mb_mgr_avx512_t2_internal(IMB_MGR *state, const int reset_mgrs)
 
         state->des_key_sched       = des_key_schedule;
 
-        state->sha1_one_block      = sha1_one_block_avx512;
-        state->sha1                = sha1_avx512;
-        state->sha224_one_block    = sha224_one_block_avx512;
-        state->sha224              = sha224_avx512;
-        state->sha256_one_block    = sha256_one_block_avx512;
-        state->sha256              = sha256_avx512;
+        state->sha1_one_block      = sha1_one_block_sse_shani;
+        state->sha1                = sha1_sse_shani;
+        state->sha224_one_block    = sha224_one_block_sse_shani;
+        state->sha224              = sha224_sse_shani;
+        state->sha256_one_block    = sha256_one_block_sse_shani;
+        state->sha256              = sha256_sse_shani;
         state->sha384_one_block    = sha384_one_block_avx512;
         state->sha384              = sha384_avx512;
         state->sha512_one_block    = sha512_one_block_avx512;
@@ -495,6 +507,7 @@ init_mb_mgr_avx512_t2_internal(IMB_MGR *state, const int reset_mgrs)
         state->md5_one_block       = md5_one_block_avx512;
 
         state->aes128_cfb_one      = aes_cfb_128_one_avx512;
+        state->aes256_cfb_one      = aes_cfb_256_one_avx512;
 
         state->eea3_1_buffer       = zuc_eea3_1_buffer_avx512;
         state->eea3_4_buffer       = zuc_eea3_4_buffer_avx;
@@ -597,6 +610,9 @@ init_mb_mgr_avx512_t2_internal(IMB_MGR *state, const int reset_mgrs)
         state->gmac128_finalize = imb_aes_gmac_finalize_128_vaes_avx512;
         state->gmac192_finalize = imb_aes_gmac_finalize_192_vaes_avx512;
         state->gmac256_finalize = imb_aes_gmac_finalize_256_vaes_avx512;
+
+        state->aes_ecb_128_quic = aes_ecb_quic_enc_128_vaes_avx512;
+        state->aes_ecb_256_quic = aes_ecb_quic_enc_256_vaes_avx512;
 }
 
 #include "mb_mgr_code.h"
