@@ -1,5 +1,5 @@
 /*******************************************************************************
- Copyright (c) 2022, Intel Corporation
+ Copyright (c) 2022-2023, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -44,7 +44,7 @@
 #include "include/cpu_feature.h"
 #include "include/aesni_emu.h"
 #include "include/error.h"
-#include "include/arch_sse_type1.h" /* snow3g */
+#include "include/arch_sse_type1.h" /* snow3g, gcm */
 #include "include/arch_avx_type1.h"
 #include "include/ooo_mgr_reset.h"
 
@@ -66,6 +66,7 @@
 #define SUBMIT_CIPHER_BURST_NOCHECK submit_cipher_burst_nocheck_avx_t1
 #define SUBMIT_HASH_BURST submit_hash_burst_avx_t1
 #define SUBMIT_HASH_BURST_NOCHECK submit_hash_burst_nocheck_avx_t1
+#define SET_SUITE_ID_FN   set_suite_id_avx_t1
 
 /* Hash */
 #define SUBMIT_JOB_HASH    SUBMIT_JOB_HASH_AVX
@@ -77,12 +78,12 @@
 #define SUBMIT_JOB_CIPHER_DEC SUBMIT_JOB_CIPHER_DEC_AVX
 
 /* AES-GCM */
-#define AES_GCM_DEC_IV_128   aes_gcm_dec_var_iv_128_avx_gen2
-#define AES_GCM_ENC_IV_128   aes_gcm_enc_var_iv_128_avx_gen2
-#define AES_GCM_DEC_IV_192   aes_gcm_dec_var_iv_192_avx_gen2
-#define AES_GCM_ENC_IV_192   aes_gcm_enc_var_iv_192_avx_gen2
-#define AES_GCM_DEC_IV_256   aes_gcm_dec_var_iv_256_avx_gen2
-#define AES_GCM_ENC_IV_256   aes_gcm_enc_var_iv_256_avx_gen2
+#define AES_GCM_DEC_IV_128   aes_gcm_dec_var_iv_128_sse
+#define AES_GCM_ENC_IV_128   aes_gcm_enc_var_iv_128_sse
+#define AES_GCM_DEC_IV_192   aes_gcm_dec_var_iv_192_sse
+#define AES_GCM_ENC_IV_192   aes_gcm_enc_var_iv_192_sse
+#define AES_GCM_DEC_IV_256   aes_gcm_dec_var_iv_256_sse
+#define AES_GCM_ENC_IV_256   aes_gcm_enc_var_iv_256_sse
 
 #define SUBMIT_JOB_AES_GCM_DEC submit_job_aes_gcm_dec_avx
 #define SUBMIT_JOB_AES_GCM_ENC submit_job_aes_gcm_enc_avx
@@ -358,6 +359,7 @@ init_mb_mgr_avx_t1_internal(IMB_MGR *state, const int reset_mgrs)
         state->submit_cipher_burst_nocheck = SUBMIT_CIPHER_BURST_NOCHECK;
         state->submit_hash_burst   = SUBMIT_HASH_BURST;
         state->submit_hash_burst_nocheck = SUBMIT_HASH_BURST_NOCHECK;
+        state->set_suite_id        = SET_SUITE_ID_FN;
 
         state->keyexp_128          = aes_keyexp_128_avx;
         state->keyexp_192          = aes_keyexp_192_avx;
@@ -382,6 +384,7 @@ init_mb_mgr_avx_t1_internal(IMB_MGR *state, const int reset_mgrs)
         state->md5_one_block       = md5_one_block_avx;
 
         state->aes128_cfb_one      = aes_cfb_128_one_avx;
+        state->aes256_cfb_one      = aes_cfb_256_one_avx;
 
         state->eea3_1_buffer       = zuc_eea3_1_buffer_avx;
         state->eea3_4_buffer       = zuc_eea3_4_buffer_avx;
@@ -434,49 +437,52 @@ init_mb_mgr_avx_t1_internal(IMB_MGR *state, const int reset_mgrs)
         state->chacha20_poly1305_dec_update = update_dec_chacha20_poly1305_avx;
         state->chacha20_poly1305_finalize = finalize_chacha20_poly1305_avx;
 
-        state->gcm128_enc          = aes_gcm_enc_128_avx_gen2;
-        state->gcm192_enc          = aes_gcm_enc_192_avx_gen2;
-        state->gcm256_enc          = aes_gcm_enc_256_avx_gen2;
-        state->gcm128_dec          = aes_gcm_dec_128_avx_gen2;
-        state->gcm192_dec          = aes_gcm_dec_192_avx_gen2;
-        state->gcm256_dec          = aes_gcm_dec_256_avx_gen2;
-        state->gcm128_init         = aes_gcm_init_128_avx_gen2;
-        state->gcm192_init         = aes_gcm_init_192_avx_gen2;
-        state->gcm256_init         = aes_gcm_init_256_avx_gen2;
-        state->gcm128_init_var_iv  = aes_gcm_init_var_iv_128_avx_gen2;
-        state->gcm192_init_var_iv  = aes_gcm_init_var_iv_192_avx_gen2;
-        state->gcm256_init_var_iv  = aes_gcm_init_var_iv_256_avx_gen2;
-        state->gcm128_enc_update   = aes_gcm_enc_128_update_avx_gen2;
-        state->gcm192_enc_update   = aes_gcm_enc_192_update_avx_gen2;
-        state->gcm256_enc_update   = aes_gcm_enc_256_update_avx_gen2;
-        state->gcm128_dec_update   = aes_gcm_dec_128_update_avx_gen2;
-        state->gcm192_dec_update   = aes_gcm_dec_192_update_avx_gen2;
-        state->gcm256_dec_update   = aes_gcm_dec_256_update_avx_gen2;
-        state->gcm128_enc_finalize = aes_gcm_enc_128_finalize_avx_gen2;
-        state->gcm192_enc_finalize = aes_gcm_enc_192_finalize_avx_gen2;
-        state->gcm256_enc_finalize = aes_gcm_enc_256_finalize_avx_gen2;
-        state->gcm128_dec_finalize = aes_gcm_dec_128_finalize_avx_gen2;
-        state->gcm192_dec_finalize = aes_gcm_dec_192_finalize_avx_gen2;
-        state->gcm256_dec_finalize = aes_gcm_dec_256_finalize_avx_gen2;
-        state->gcm128_precomp      = aes_gcm_precomp_128_avx_gen2;
-        state->gcm192_precomp      = aes_gcm_precomp_192_avx_gen2;
-        state->gcm256_precomp      = aes_gcm_precomp_256_avx_gen2;
-        state->gcm128_pre          = aes_gcm_pre_128_avx_gen2;
-        state->gcm192_pre          = aes_gcm_pre_192_avx_gen2;
-        state->gcm256_pre          = aes_gcm_pre_256_avx_gen2;
+        state->gcm128_enc          = aes_gcm_enc_128_sse;
+        state->gcm192_enc          = aes_gcm_enc_192_sse;
+        state->gcm256_enc          = aes_gcm_enc_256_sse;
+        state->gcm128_dec          = aes_gcm_dec_128_sse;
+        state->gcm192_dec          = aes_gcm_dec_192_sse;
+        state->gcm256_dec          = aes_gcm_dec_256_sse;
+        state->gcm128_init         = aes_gcm_init_128_sse;
+        state->gcm192_init         = aes_gcm_init_192_sse;
+        state->gcm256_init         = aes_gcm_init_256_sse;
+        state->gcm128_init_var_iv  = aes_gcm_init_var_iv_128_sse;
+        state->gcm192_init_var_iv  = aes_gcm_init_var_iv_192_sse;
+        state->gcm256_init_var_iv  = aes_gcm_init_var_iv_256_sse;
+        state->gcm128_enc_update   = aes_gcm_enc_128_update_sse;
+        state->gcm192_enc_update   = aes_gcm_enc_192_update_sse;
+        state->gcm256_enc_update   = aes_gcm_enc_256_update_sse;
+        state->gcm128_dec_update   = aes_gcm_dec_128_update_sse;
+        state->gcm192_dec_update   = aes_gcm_dec_192_update_sse;
+        state->gcm256_dec_update   = aes_gcm_dec_256_update_sse;
+        state->gcm128_enc_finalize = aes_gcm_enc_128_finalize_sse;
+        state->gcm192_enc_finalize = aes_gcm_enc_192_finalize_sse;
+        state->gcm256_enc_finalize = aes_gcm_enc_256_finalize_sse;
+        state->gcm128_dec_finalize = aes_gcm_dec_128_finalize_sse;
+        state->gcm192_dec_finalize = aes_gcm_dec_192_finalize_sse;
+        state->gcm256_dec_finalize = aes_gcm_dec_256_finalize_sse;
+        state->gcm128_precomp      = aes_gcm_precomp_128_sse;
+        state->gcm192_precomp      = aes_gcm_precomp_192_sse;
+        state->gcm256_precomp      = aes_gcm_precomp_256_sse;
+        state->gcm128_pre          = aes_gcm_pre_128_sse;
+        state->gcm192_pre          = aes_gcm_pre_192_sse;
+        state->gcm256_pre          = aes_gcm_pre_256_sse;
 
-        state->ghash               = ghash_avx_gen2;
-        state->ghash_pre           = ghash_pre_avx_gen2;
+        state->ghash               = ghash_sse;
+        state->ghash_pre           = ghash_pre_sse;
 
-        state->gmac128_init        = imb_aes_gmac_init_128_avx_gen2;
-        state->gmac192_init        = imb_aes_gmac_init_192_avx_gen2;
-        state->gmac256_init        = imb_aes_gmac_init_256_avx_gen2;
-        state->gmac128_update      = imb_aes_gmac_update_128_avx_gen2;
-        state->gmac192_update      = imb_aes_gmac_update_192_avx_gen2;
-        state->gmac256_update      = imb_aes_gmac_update_256_avx_gen2;
-        state->gmac128_finalize    = imb_aes_gmac_finalize_128_avx_gen2;
-        state->gmac192_finalize    = imb_aes_gmac_finalize_192_avx_gen2;
-        state->gmac256_finalize    = imb_aes_gmac_finalize_256_avx_gen2;
+        state->gmac128_init        = imb_aes_gmac_init_128_sse;
+        state->gmac192_init        = imb_aes_gmac_init_192_sse;
+        state->gmac256_init        = imb_aes_gmac_init_256_sse;
+        state->gmac128_update      = imb_aes_gmac_update_128_sse;
+        state->gmac192_update      = imb_aes_gmac_update_192_sse;
+        state->gmac256_update      = imb_aes_gmac_update_256_sse;
+        state->gmac128_finalize    = imb_aes_gmac_finalize_128_sse;
+        state->gmac192_finalize    = imb_aes_gmac_finalize_192_sse;
+        state->gmac256_finalize    = imb_aes_gmac_finalize_256_sse;
+
+        state->aes_ecb_128_quic    = aes_ecb_quic_enc_128_avx;
+        state->aes_ecb_256_quic    = aes_ecb_quic_enc_256_avx;
 }
 
 #include "mb_mgr_code.h"
