@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2018-2022, Intel Corporation
+  Copyright (c) 2018-2023, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -517,7 +517,6 @@ aes_gcm_init_var_iv_256_sse(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
                             const uint8_t *iv, const uint64_t iv_len,
                             const uint8_t *aad, const uint64_t aad_len);
-
 IMB_DLL_EXPORT void
 aes_gcm_init_var_iv_256_avx_gen2(const struct gcm_key_data *key_data,
                                  struct gcm_context_data *context_data,
@@ -533,312 +532,76 @@ aes_gcm_init_var_iv_256_avx_gen4(const struct gcm_key_data *key_data,
  * Internal GCM API for SSE/AVX/AVX2/AVX512/AESNI emulation,
  * to be used only through job API.
  */
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_128_sse(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_192_sse(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_256_sse(IMB_MGR *state, IMB_JOB *job);
 
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_128_sse(const struct gcm_key_data *key_data,
-                           struct gcm_context_data *context_data,
-                           uint8_t *out, const uint8_t *in,
-                           const uint64_t msg_len,
-                           const uint8_t *iv, const uint64_t iv_len,
-                           const uint8_t *aad, const uint64_t aad_len,
-                           uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_192_sse(const struct gcm_key_data *key_data,
-                           struct gcm_context_data *context_data,
-                           uint8_t *out, const uint8_t *in,
-                           const uint64_t msg_len,
-                           const uint8_t *iv, const uint64_t iv_len,
-                           const uint8_t *aad, const uint64_t aad_len,
-                           uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_256_sse(const struct gcm_key_data *key_data,
-                           struct gcm_context_data *context_data,
-                           uint8_t *out, const uint8_t *in,
-                           const uint64_t msg_len,
-                           const uint8_t *iv, const uint64_t iv_len,
-                           const uint8_t *aad, const uint64_t aad_len,
-                           uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_128_sse(const struct gcm_key_data *key_data,
-                           struct gcm_context_data *context_data,
-                           uint8_t *out, const uint8_t *in,
-                           const uint64_t msg_len,
-                           const uint8_t *iv, const uint64_t iv_len,
-                           const uint8_t *aad, const uint64_t aad_len,
-                           uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_192_sse(const struct gcm_key_data *key_data,
-                           struct gcm_context_data *context_data,
-                           uint8_t *out, const uint8_t *in,
-                           const uint64_t msg_len,
-                           const uint8_t *iv, const uint64_t iv_len,
-                           const uint8_t *aad, const uint64_t aad_len,
-                           uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_256_sse(const struct gcm_key_data *key_data,
-                           struct gcm_context_data *context_data,
-                           uint8_t *out, const uint8_t *in,
-                           const uint64_t msg_len,
-                           const uint8_t *iv, const uint64_t iv_len,
-                           const uint8_t *aad, const uint64_t aad_len,
-                           uint8_t *auth_tag, const uint64_t auth_tag_len);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_128_sse(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_192_sse(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_256_sse(IMB_MGR *state, IMB_JOB *job);
 
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_128_avx_gen2(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_192_avx_gen2(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_256_avx_gen2(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_128_avx_gen2(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_192_avx_gen2(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_256_avx_gen2(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_128_avx_gen4(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_192_avx_gen4(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_256_avx_gen4(IMB_MGR *state, IMB_JOB *job);
 
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_128_avx_gen4(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_192_avx_gen4(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_256_avx_gen4(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_128_avx_gen4(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_192_avx_gen4(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_256_avx_gen4(const struct gcm_key_data *key_data,
-                                struct gcm_context_data *context_data,
-                                uint8_t *out, const uint8_t *in,
-                                const uint64_t msg_len,
-                                const uint8_t *iv, const uint64_t iv_len,
-                                const uint8_t *aad, const uint64_t aad_len,
-                                uint8_t *auth_tag, const uint64_t auth_tag_len);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_128_avx_gen4(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_192_avx_gen4(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_256_avx_gen4(IMB_MGR *state, IMB_JOB *job);
 
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_128_avx512(const struct gcm_key_data *key_data,
-                              struct gcm_context_data *context_data,
-                              uint8_t *out, const uint8_t *in,
-                              const uint64_t msg_len,
-                              const uint8_t *iv, const uint64_t iv_len,
-                              const uint8_t *aad, const uint64_t aad_len,
-                              uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_192_avx512(const struct gcm_key_data *key_data,
-                              struct gcm_context_data *context_data,
-                              uint8_t *out, const uint8_t *in,
-                              const uint64_t msg_len,
-                              const uint8_t *iv, const uint64_t iv_len,
-                              const uint8_t *aad, const uint64_t aad_len,
-                              uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_256_avx512(const struct gcm_key_data *key_data,
-                              struct gcm_context_data *context_data,
-                              uint8_t *out, const uint8_t *in,
-                              const uint64_t msg_len,
-                              const uint8_t *iv, const uint64_t iv_len,
-                              const uint8_t *aad, const uint64_t aad_len,
-                              uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_128_avx512(const struct gcm_key_data *key_data,
-                              struct gcm_context_data *context_data,
-                              uint8_t *out, const uint8_t *in,
-                              const uint64_t msg_len,
-                              const uint8_t *iv, const uint64_t iv_len,
-                              const uint8_t *aad, const uint64_t aad_len,
-                              uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_192_avx512(const struct gcm_key_data *key_data,
-                              struct gcm_context_data *context_data,
-                              uint8_t *out, const uint8_t *in,
-                              const uint64_t msg_len,
-                              const uint8_t *iv, const uint64_t iv_len,
-                              const uint8_t *aad, const uint64_t aad_len,
-                              uint8_t *auth_tag, const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_256_avx512(const struct gcm_key_data *key_data,
-                              struct gcm_context_data *context_data,
-                              uint8_t *out, const uint8_t *in,
-                              const uint64_t msg_len,
-                              const uint8_t *iv, const uint64_t iv_len,
-                              const uint8_t *aad, const uint64_t aad_len,
-                              uint8_t *auth_tag, const uint64_t auth_tag_len);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_128_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_192_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_256_avx512(IMB_MGR *state, IMB_JOB *job);
 
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_128_vaes_avx512(const struct gcm_key_data *key_data,
-                                   struct gcm_context_data *context_data,
-                                   uint8_t *out, const uint8_t *in,
-                                   const uint64_t msg_len,
-                                   const uint8_t *iv, const uint64_t iv_len,
-                                   const uint8_t *aad, const uint64_t aad_len,
-                                   uint8_t *auth_tag,
-                                   const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_192_vaes_avx512(const struct gcm_key_data *key_data,
-                                   struct gcm_context_data *context_data,
-                                   uint8_t *out, const uint8_t *in,
-                                   const uint64_t msg_len,
-                                   const uint8_t *iv, const uint64_t iv_len,
-                                   const uint8_t *aad, const uint64_t aad_len,
-                                   uint8_t *auth_tag,
-                                   const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_256_vaes_avx512(const struct gcm_key_data *key_data,
-                                   struct gcm_context_data *context_data,
-                                   uint8_t *out, const uint8_t *in,
-                                   const uint64_t msg_len,
-                                   const uint8_t *iv, const uint64_t iv_len,
-                                   const uint8_t *aad, const uint64_t aad_len,
-                                   uint8_t *auth_tag,
-                                   const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_128_vaes_avx512(const struct gcm_key_data *key_data,
-                                   struct gcm_context_data *context_data,
-                                   uint8_t *out, const uint8_t *in,
-                                   const uint64_t msg_len,
-                                   const uint8_t *iv, const uint64_t iv_len,
-                                   const uint8_t *aad, const uint64_t aad_len,
-                                   uint8_t *auth_tag,
-                                   const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_192_vaes_avx512(const struct gcm_key_data *key_data,
-                                   struct gcm_context_data *context_data,
-                                   uint8_t *out, const uint8_t *in,
-                                   const uint64_t msg_len,
-                                   const uint8_t *iv, const uint64_t iv_len,
-                                   const uint8_t *aad, const uint64_t aad_len,
-                                   uint8_t *auth_tag,
-                                   const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_256_vaes_avx512(const struct gcm_key_data *key_data,
-                                   struct gcm_context_data *context_data,
-                                   uint8_t *out, const uint8_t *in,
-                                   const uint64_t msg_len,
-                                   const uint8_t *iv, const uint64_t iv_len,
-                                   const uint8_t *aad, const uint64_t aad_len,
-                                   uint8_t *auth_tag,
-                                   const uint64_t auth_tag_len);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_128_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_192_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_256_avx512(IMB_MGR *state, IMB_JOB *job);
+
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_128_vaes_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_192_vaes_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_256_vaes_avx512(IMB_MGR *state, IMB_JOB *job);
+
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_128_vaes_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_192_vaes_avx512(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_256_vaes_avx512(IMB_MGR *state, IMB_JOB *job);
+
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_128_sse_no_aesni(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_192_sse_no_aesni(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_enc_var_iv_256_sse_no_aesni(IMB_MGR *state, IMB_JOB *job);
+
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_128_sse_no_aesni(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_192_sse_no_aesni(IMB_MGR *state, IMB_JOB *job);
+IMB_DLL_LOCAL IMB_JOB *
+aes_gcm_dec_var_iv_256_sse_no_aesni(IMB_MGR *state, IMB_JOB *job);
 
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_128_sse_no_aesni(const struct gcm_key_data *key_data,
-                                    struct gcm_context_data *context_data,
-                                    uint8_t *out, const uint8_t *in,
-                                    const uint64_t msg_len,
-                                    const uint8_t *iv, const uint64_t iv_len,
-                                    const uint8_t *aad, const uint64_t aad_len,
-                                    uint8_t *auth_tag,
-                                    const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_192_sse_no_aesni(const struct gcm_key_data *key_data,
-                                    struct gcm_context_data *context_data,
-                                    uint8_t *out, const uint8_t *in,
-                                    const uint64_t msg_len,
-                                    const uint8_t *iv, const uint64_t iv_len,
-                                    const uint8_t *aad, const uint64_t aad_len,
-                                    uint8_t *auth_tag,
-                                    const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_enc_var_iv_256_sse_no_aesni(const struct gcm_key_data *key_data,
-                                    struct gcm_context_data *context_data,
-                                    uint8_t *out, const uint8_t *in,
-                                    const uint64_t msg_len,
-                                    const uint8_t *iv, const uint64_t iv_len,
-                                    const uint8_t *aad, const uint64_t aad_len,
-                                    uint8_t *auth_tag,
-                                    const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_128_sse_no_aesni(const struct gcm_key_data *key_data,
-                                    struct gcm_context_data *context_data,
-                                    uint8_t *out, const uint8_t *in,
-                                    const uint64_t msg_len,
-                                    const uint8_t *iv, const uint64_t iv_len,
-                                    const uint8_t *aad, const uint64_t aad_len,
-                                    uint8_t *auth_tag,
-                                    const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_192_sse_no_aesni(const struct gcm_key_data *key_data,
-                                    struct gcm_context_data *context_data,
-                                    uint8_t *out, const uint8_t *in,
-                                    const uint64_t msg_len,
-                                    const uint8_t *iv, const uint64_t iv_len,
-                                    const uint8_t *aad, const uint64_t aad_len,
-                                    uint8_t *auth_tag,
-                                    const uint64_t auth_tag_len);
-IMB_DLL_LOCAL void
-aes_gcm_dec_var_iv_256_sse_no_aesni(const struct gcm_key_data *key_data,
-                                    struct gcm_context_data *context_data,
-                                    uint8_t *out, const uint8_t *in,
-                                    const uint64_t msg_len,
-                                    const uint8_t *iv, const uint64_t iv_len,
-                                    const uint8_t *aad, const uint64_t aad_len,
-                                    uint8_t *auth_tag,
-                                    const uint64_t auth_tag_len);
 /*
  * GHASH API for SSE/AVX/AVX2/AVX512/AESNI emulation
  */
@@ -875,6 +638,12 @@ ghash_pre_sse(const void *key, struct gcm_key_data *key_data);
 IMB_DLL_EXPORT void
 ghash_pre_avx_gen2(const void *key, struct gcm_key_data *key_data);
 
+IMB_DLL_EXPORT void
+ghash_pre_avx_gen4(const void *key, struct gcm_key_data *key_data);
+
+IMB_DLL_EXPORT void
+ghash_pre_avx512(const void *key, struct gcm_key_data *key_data);
+
 IMB_DLL_EXPORT void
 ghash_pre_vaes_avx512(const void *key, struct gcm_key_data *key_data);
 
