@@ -1,5 +1,5 @@
 /*******************************************************************************
- Copyright (c) 2020-2022, Intel Corporation
+ Copyright (c) 2020-2023, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -37,4 +37,7 @@ void memcpy_fn_avx_16(void *dst, const void *src, const size_t size);
 /* Memcpy 128 bytes with SSE instructions */
 void memcpy_fn_sse_128(void *dst, const void *src);
 
+/* Basic memcpy that doesn't use stack */
+void safe_memcpy(void *dst, const void *src, const size_t size);
+
 #endif /* MEMCPY_H */
