@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2009-2022, Intel Corporation
+  Copyright (c) 2009-2023, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -314,6 +314,7 @@ IMB_DLL_LOCAL void asm_Zuc256Initialization_4_gfni_sse(ZucKey4_t *pKeys,
 IMB_DLL_LOCAL void asm_Zuc256Initialization_4_avx(ZucKey4_t *pKeys,
                                                   const uint8_t *ivs,
                                                   ZucState4_t *pState,
+                                                  void *tags,
                                                   const uint64_t tag_sz);
 
 
@@ -341,6 +342,9 @@ IMB_DLL_LOCAL void asm_Zuc256Initialization_4_avx(ZucKey4_t *pKeys,
 IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
                                                 const uint8_t *ivs,
                                                 ZucState8_t *pState);
+IMB_DLL_LOCAL void asm_ZucInitialization_8_gfni_avx2(ZucKey8_t *pKeys,
+                                                const uint8_t *ivs,
+                                                ZucState8_t *pState);
 
 /**
  ******************************************************************************
@@ -360,6 +364,12 @@ IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
  * @param[in,out] pState            Pointer to a ZUC state structure of type
  *                                  @ref ZucState8_t that will be populated
  *                                  with the initialized ZUC state.
+ * @param[in,out] tags              Array of 4 pointers to authentication tags
+ *                                  (up to 16 bytes each, only for ZUC-EIA3)
+ * @param[in] tag_sz                Tag size (0, 4, 8 or 16), to select the
+ *                                  constants used to initialize the LFSR
+ *                                  the LFSR registers (0 is used in case of
+ *                                  encryption).
  *
  * @pre
  *      None
@@ -368,7 +378,13 @@ IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
 IMB_DLL_LOCAL void asm_Zuc256Initialization_8_avx2(ZucKey8_t *pKeys,
                                                    const uint8_t *ivs,
                                                    ZucState8_t *pState,
-                                                   const unsigned tag_sz);
+                                                   void *tags,
+                                                   const uint64_t tag_sz);
+IMB_DLL_LOCAL void asm_Zuc256Initialization_8_gfni_avx2(ZucKey8_t *pKeys,
+                                                   const uint8_t *ivs,
+                                                   ZucState8_t *pState,
+                                                   void *tags,
+                                                   const uint64_t tag_sz);
 
 /**
  ******************************************************************************
@@ -625,6 +641,8 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream16B_4_avx(ZucState4_t *pState,
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucGenKeystream32B_8_avx2(ZucState8_t *pState,
                                                  uint32_t *pKeyStr[8]);
+IMB_DLL_LOCAL void asm_ZucGenKeystream32B_8_gfni_avx2(ZucState8_t *pState,
+                                                 uint32_t *pKeyStr[8]);
 
 /**
  ******************************************************************************
@@ -655,6 +673,44 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_avx512(ZucState16_t *pState,
 IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_gfni_avx512(ZucState16_t *pState,
                                                         uint32_t *pKeyStr,
                                                         const unsigned key_off);
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 64 bytes of
+ *      keystream for 16 packets in parallel, except for selected lanes,
+ *      which will have 48 bytes of keystream generated instead.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 16 input buffers
+ *                                  that will contain the generated keystream
+ *                                  for these 16 packets.
+ *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
+ * @param[in] lane_mask             Mask containing lanes which will have 64
+ *                                  bytes of KS generated (16 bytes less for
+ *                                  the rest)
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      the ZUC state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream64B_16_skip16_avx512(ZucState16_t *pState,
+                                        uint32_t *pKeyStr,
+                                        const unsigned key_off,
+                                        const uint16_t lane_mask);
+
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream64B_16_skip16_gfni_avx512(ZucState16_t *pState,
+                                             uint32_t *pKeyStr,
+                                             const unsigned key_off,
+                                             const uint16_t lane_mask);
 /**
  ******************************************************************************
  *
@@ -682,16 +738,55 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_gfni_avx512(ZucState16_t *pState,
  *      the ZUC state.
  *
  *****************************************************************************/
-IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_skip8_avx512(ZucState16_t *pState,
-                                                    uint32_t *pKeyStr,
-                                                    const unsigned key_off,
-                                                    const uint16_t lane_mask);
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream64B_16_skip8_avx512(ZucState16_t *pState,
+                                       uint32_t *pKeyStr,
+                                       const unsigned key_off,
+                                       const uint16_t lane_mask);
 
 IMB_DLL_LOCAL void
 asm_ZucGenKeystream64B_16_skip8_gfni_avx512(ZucState16_t *pState,
                                             uint32_t *pKeyStr,
                                             const unsigned key_off,
                                             const uint16_t lane_mask);
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 64 bytes of
+ *      keystream for 16 packets in parallel, except for selected lanes,
+ *      which will have 60 bytes of keystream generated instead.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 16 input buffers
+ *                                  that will contain the generated keystream
+ *                                  for these 16 packets.
+ *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
+ * @param[in] lane_mask             Mask containing lanes which will have 64
+ *                                  bytes of KS generated (4 bytes less for
+ *                                  the rest)
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      the ZUC state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream64B_16_skip4_avx512(ZucState16_t *pState,
+                                       uint32_t *pKeyStr,
+                                       const unsigned key_off,
+                                       const uint16_t lane_mask);
+
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream64B_16_skip4_gfni_avx512(ZucState16_t *pState,
+                                            uint32_t *pKeyStr,
+                                            const unsigned key_off,
+                                            const uint16_t lane_mask);
 /**
  ******************************************************************************
  *
@@ -756,6 +851,31 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_gfni_sse(ZucState4_t *pState,
 IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_avx(ZucState4_t *pState,
                                                uint32_t *pKeyStr[4]);
 
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 16 bytes of
+ *      keystream for eight packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState8_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 8 input buffers that
+ *                                  will contain the generated keystream for
+ *                                  these 8 packets.
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_8 to initialize the ZUC
+ *      state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_ZucGenKeystream16B_8_avx2(ZucState8_t *pState,
+                                                uint32_t *pKeyStr[8]);
+IMB_DLL_LOCAL void asm_ZucGenKeystream16B_8_gfni_avx2(ZucState8_t *pState,
+                                                uint32_t *pKeyStr[8]);
+
 /**
  ******************************************************************************
  *
@@ -778,6 +898,8 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_avx(ZucState4_t *pState,
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucGenKeystream8B_8_avx2(ZucState8_t *pState,
                                                 uint32_t *pKeyStr[8]);
+IMB_DLL_LOCAL void asm_ZucGenKeystream8B_8_gfni_avx2(ZucState8_t *pState,
+                                                uint32_t *pKeyStr[8]);
 
 /**
  ******************************************************************************
@@ -801,6 +923,8 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream8B_8_avx2(ZucState8_t *pState,
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucGenKeystream4B_8_avx2(ZucState8_t *pState,
                                                 uint32_t *pKeyStr[8]);
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_8_gfni_avx2(ZucState8_t *pState,
+                                                uint32_t *pKeyStr[8]);
 
 /**
  ******************************************************************************
@@ -832,36 +956,6 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream8B_16_gfni_avx512(ZucState16_t *pState,
                                                         uint32_t *pKeyStr,
                                                         const unsigned key_off);
 
-/**
- ******************************************************************************
- *
- * @description
- *      Definition of the external function that implements the working
- *      stage of the ZUC algorithm. The function will generate 4 bytes of
- *      keystream for sixteen packets in parallel.
- *
- * @param[in] pState                Pointer to a ZUC state structure of type
- *                                  @ref ZucState16_t
- *
- * @param[in,out] pKeyStr           Pointer to buffer to write consecutively 4
- *                                  bytes of keystream for the 16 input buffers
- *
- * @param[in] lane_mask             Mask containing lanes which will have 4
- *                                  bytes of KS generated (no bytes generated
- *                                  for the rest)
- * @pre
- *      A successful call to @ref asm_ZucInitialization_16 to initialize the ZUC
- *      state.
- *
- *****************************************************************************/
-IMB_DLL_LOCAL void asm_ZucGenKeystream4B_16_avx512(ZucState16_t *pState,
-                                                   uint32_t pKeyStr[16],
-                                                   const uint32_t lane_mask);
-
-IMB_DLL_LOCAL void asm_ZucGenKeystream4B_16_gfni_avx512(ZucState16_t *pState,
-                                                      uint32_t pKeyStr[16],
-                                                      const uint32_t lane_mask);
-
 /**
  ******************************************************************************
  *
@@ -897,6 +991,47 @@ asm_ZucGenKeystream_16_gfni_avx512(ZucState16_t *pState,
                                    uint32_t *pKstr,
                                    const unsigned key_off,
                                    const uint32_t numRounds);
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate N*4 bytes of
+ *      keystream for sixteen packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 16 input buffers
+ *                                  that will contain the generated keystream
+ *                                  for these 16 packets.
+ *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
+ * @param[in] lane_mask             Mask containing lanes which will have N*4
+ *                                  bytes of KS generated (16 bytes less for
+ *                                  the rest)
+ *
+ * @param[in] numRounds             Number of 4-byte rounds (1 to 16 rounds)
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      ZUC state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_skip16_avx512(ZucState16_t *pState,
+                                    uint32_t *pKstr,
+                                    const unsigned key_off,
+                                    const uint16_t lane_mask,
+                                    const uint32_t numRounds);
+
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_skip16_gfni_avx512(ZucState16_t *pState,
+                                         uint32_t *pKstr,
+                                         const unsigned key_off,
+                                         const uint16_t lane_mask,
+                                         const uint32_t numRounds);
 
 /**
  ******************************************************************************
@@ -922,8 +1057,8 @@ asm_ZucGenKeystream_16_gfni_avx512(ZucState16_t *pState,
  * @param[in] numRounds             Number of 4-byte rounds (1 to 16 rounds)
  *
  * @pre
- *      A successful call to @ref asm_ZucInitialization to initialize the ZUC
- *      state.
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      ZUC state.
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void
@@ -939,6 +1074,48 @@ asm_ZucGenKeystream_16_skip8_gfni_avx512(ZucState16_t *pState,
                                          const unsigned key_off,
                                          const uint16_t lane_mask,
                                          const uint32_t numRounds);
+
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate N*4 bytes of
+ *      keystream for sixteen packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 16 input buffers
+ *                                  that will contain the generated keystream
+ *                                  for these 16 packets.
+ *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
+ * @param[in] lane_mask             Mask containing lanes which will have N*4
+ *                                  bytes of KS generated (4 bytes less for
+ *                                  the rest)
+ *
+ * @param[in] numRounds             Number of 4-byte rounds (1 to 16 rounds)
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      ZUC state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_skip4_avx512(ZucState16_t *pState,
+                                    uint32_t *pKstr,
+                                    const unsigned key_off,
+                                    const uint16_t lane_mask,
+                                    const uint32_t numRounds);
+
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_skip4_gfni_avx512(ZucState16_t *pState,
+                                         uint32_t *pKstr,
+                                         const unsigned key_off,
+                                         const uint16_t lane_mask,
+                                         const uint32_t numRounds);
 /**
  ******************************************************************************
  *
@@ -1018,6 +1195,11 @@ IMB_DLL_LOCAL void asm_ZucCipher_8_avx2(ZucState8_t *pState,
                                         uint64_t *pOut[8],
                                         const uint16_t lengths[8],
                                         const uint64_t minLength);
+IMB_DLL_LOCAL void asm_ZucCipher_8_gfni_avx2(ZucState8_t *pState,
+                                             const uint64_t *pIn[8],
+                                             uint64_t *pOut[8],
+                                             const uint16_t lengths[8],
+                                             const uint64_t minLength);
 
 /**
  ******************************************************************************
@@ -1073,12 +1255,25 @@ IMB_DLL_LOCAL void asm_ZucCipher_16_gfni_avx512(ZucState16_t *pState,
  *      None
  *
  *****************************************************************************/
-IMB_DLL_LOCAL void asm_Eia3Round16BSSE(void *T, const void *ks,
-                                       const void *data, const uint64_t tag_sz);
+IMB_DLL_LOCAL void asm_Eia3Round16B_sse(void *T, const void *ks,
+                                        const void *data,
+                                        const uint64_t tag_sz);
 
-IMB_DLL_LOCAL void asm_Eia3Round16BSSE_no_aesni(void *T, const void *ks,
-                                                const void *data,
-                                                const uint64_t tag_sz);
+IMB_DLL_LOCAL void asm_Eia3Round16B_sse_no_aesni(void *T, const void *ks,
+                                                 const void *data,
+                                                 const uint64_t tag_sz);
+
+IMB_DLL_LOCAL void asm_Eia3Round16B_gfni_sse(void *T, const void *ks,
+                                             const void *data,
+                                             const uint64_t tag_sz);
+
+IMB_DLL_LOCAL void asm_Eia3Round16B_avx(void *T, const void *ks,
+                                        const void *data,
+                                        const uint64_t tag_sz);
+
+IMB_DLL_LOCAL void asm_Eia3Round32B_avx(void *T, const void *ks,
+                                        const void *data,
+                                        const uint64_t tag_sz);
 
 /**
  ******************************************************************************
@@ -1096,17 +1291,29 @@ IMB_DLL_LOCAL void asm_Eia3Round16BSSE_no_aesni(void *T, const void *ks,
  *      None
  *
  *****************************************************************************/
-IMB_DLL_LOCAL void asm_Eia3RemainderSSE(void *T, const void *ks,
-                                        const void *data,
-                                        const uint64_t n_bits,
-                                        const uint64_t key_size,
-                                        const uint64_t tag_size);
+IMB_DLL_LOCAL void asm_Eia3Remainder_sse(void *T, const void *ks,
+                                         const void *data,
+                                         const uint64_t n_bits,
+                                         const uint64_t key_size,
+                                         const uint64_t tag_size);
 
-IMB_DLL_LOCAL void asm_Eia3RemainderSSE_no_aesni(void *T, const void *ks,
-                                                 const void *data,
-                                                 const uint64_t n_bits,
-                                                 const uint64_t key_size,
-                                                 const uint64_t tag_size);
+IMB_DLL_LOCAL void asm_Eia3Remainder_sse_no_aesni(void *T, const void *ks,
+                                                  const void *data,
+                                                  const uint64_t n_bits,
+                                                  const uint64_t key_size,
+                                                  const uint64_t tag_size);
+
+IMB_DLL_LOCAL void asm_Eia3Remainder_gfni_sse(void *T, const void *ks,
+                                              const void *data,
+                                              const uint64_t n_bits,
+                                              const uint64_t key_size,
+                                              const uint64_t tag_size);
+
+IMB_DLL_LOCAL void asm_Eia3Remainder_avx(void *T, const void *ks,
+                                         const void *data,
+                                         const uint64_t n_bits,
+                                         const uint64_t key_size,
+                                         const uint64_t tag_size);
 
 /**
  ******************************************************************************
@@ -1114,57 +1321,36 @@ IMB_DLL_LOCAL void asm_Eia3RemainderSSE_no_aesni(void *T, const void *ks,
  *      Definition of the external function to update the authentication tag
  *      based on keystream and data (AVX variant)
  *
- * @param[in] T                     Authentication tag
+ * @param[in] T                     Array of authentication tags for 16 buffers
  *
- * @param[in] ks                    Pointer to key stream
+ * @param[in] ks                    Array of keystreams for 16 buffers
  *
- * @param[in] data                  Pointer to the data
+ * @param[in] data                  Array of pointers to the data for 16 buffers
+ *
+ * @param[in] len                   Array of remaining lengths for 16 buffers
+ *
+ * @param[in] tag_sz                Tag size (4, 8 or 16 bytes)
  *
  * @pre
  *      None
  *
  *****************************************************************************/
-IMB_DLL_LOCAL uint32_t asm_Eia3Round64BAVX(uint32_t T, const void *ks,
-                                           const void *data);
 
-IMB_DLL_LOCAL void asm_Eia3Round64BAVX512_16(uint32_t *T,
+IMB_DLL_LOCAL void asm_Eia3Round64BAVX512_16(void *T,
                                              const uint32_t *ks,
                                              const void **data,
-                                             uint16_t *len);
+                                             uint16_t *len,
+                                             const uint64_t tag_sz);
 
-IMB_DLL_LOCAL void asm_Eia3Round64B_16_VPCLMUL(uint32_t *T,
+IMB_DLL_LOCAL void asm_Eia3Round64B_16_VPCLMUL(void *T,
                                                const uint32_t *ks,
                                                const void **data,
-                                               uint16_t *len);
-
-IMB_DLL_LOCAL uint32_t asm_Eia3Round32BAVX(uint32_t T, const void *ks,
-                                           const void *data);
-
-IMB_DLL_LOCAL uint32_t asm_Eia3Round16BAVX(uint32_t T, const void *ks,
-                                           const void *data);
+                                               uint16_t *len,
+                                               const uint64_t tag_sz);
 
 IMB_DLL_LOCAL void asm_Eia3Round64BAVX512(uint32_t *T, const void *ks,
                                           const void *data);
 
-/**
- ******************************************************************************
- * @description
- *      Definition of the external function to return the authentication
- *      update value to be XOR'ed with current authentication tag (AVX variant)
- *
- * @param[in] ks                    Pointer to key stream
- *
- * @param[in] data                  Pointer to the data
- *
- * @param[in] n_words               Number of data bits to be processed
- *
- * @pre
- *      None
- *
- *****************************************************************************/
-IMB_DLL_LOCAL uint32_t asm_Eia3RemainderAVX(const void *ks, const void *data,
-                                            const uint64_t n_words);
-
 /**
  ******************************************************************************
  * @description
@@ -1197,7 +1383,8 @@ IMB_DLL_LOCAL uint32_t asm_Eia3_256_RemainderAVX512_16(uint32_t *T,
                                                   const uint32_t *ks,
                                                   const void **data,
                                                   uint16_t *lens,
-                                                  const uint32_t commonBits);
+                                                  const uint64_t commonBits,
+                                                  const uint64_t tag_size);
 
 IMB_DLL_LOCAL uint32_t asm_Eia3RemainderAVX512_16_VPCLMUL(uint32_t *T,
                                                   const uint32_t *ks,
@@ -1209,7 +1396,8 @@ IMB_DLL_LOCAL uint32_t asm_Eia3_256_RemainderAVX512_16_VPCLMUL(uint32_t *T,
                                                   const uint32_t *ks,
                                                   const void **data,
                                                   uint16_t *lens,
-                                                  const uint32_t commonBits);
+                                                  const uint64_t commonBits,
+                                                  const uint64_t tag_size);
 
 /**
  ******************************************************************************
@@ -1222,6 +1410,7 @@ IMB_DLL_LOCAL uint32_t asm_Eia3_256_RemainderAVX512_16_VPCLMUL(uint32_t *T,
  * @param[in] data                  Array of 16 pointers to data for 16 buffers
  * @param[in] len                   Array of lengths for 16 buffers
  * @param[in] numRounds             Number of 64B rounds to perform
+ * @param[in] tagSize               Tag size (4 or 8 bytes)
  *
  *****************************************************************************/
 IMB_DLL_LOCAL
@@ -1230,14 +1419,16 @@ void asm_Eia3_Nx64B_AVX512_16(ZucState16_t *pState,
                               uint32_t *T,
                               const void **data,
                               uint16_t *len,
-                              const uint32_t numRounds);
+                              const uint64_t numRounds,
+                              const uint64_t tagSize);
 IMB_DLL_LOCAL
 void asm_Eia3_Nx64B_AVX512_16_VPCLMUL(ZucState16_t *pState,
                                       uint32_t *pKeyStr,
                                       uint32_t *T,
                                       const void **data,
                                       uint16_t *len,
-                                      const uint32_t numRounds);
+                                      const uint64_t numRounds,
+                                      const uint64_t tagSize);
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_gfni_sse(const void * const pKey[4],
                                     const uint8_t *ivs,
@@ -1301,9 +1492,10 @@ IMB_DLL_LOCAL
 void zuc256_eia3_4_buffer_job_avx(const void * const pKey[4],
                                const uint8_t *ivs,
                                const void * const pBufferIn[4],
-                               uint32_t *pMacI[4],
+                               void *pMacI[4],
                                const uint16_t lengthInBits[4],
-                               const void * const job_in_lane[4]);
+                               const void * const job_in_lane[4],
+                               const uint64_t tag_size);
 
 IMB_DLL_LOCAL
 void zuc_eia3_8_buffer_job_avx2(const void * const pKey[8],
@@ -1313,14 +1505,31 @@ void zuc_eia3_8_buffer_job_avx2(const void * const pKey[8],
                                 const uint16_t lengthInBits[8],
                                 const void * const job_in_lane[8]);
 
+IMB_DLL_LOCAL
+void zuc_eia3_8_buffer_job_gfni_avx2(const void * const pKey[8],
+                                const uint8_t *ivs,
+                                const void * const pBufferIn[8],
+                                uint32_t *pMacI[8],
+                                const uint16_t lengthInBits[8],
+                                const void * const job_in_lane[8]);
+
 IMB_DLL_LOCAL
 void zuc256_eia3_8_buffer_job_avx2(const void * const pKey[8],
                                    const uint8_t *ivs,
                                    const void * const pBufferIn[8],
-                                   uint32_t *pMacI[8],
+                                   void *pMacI[8],
                                    const uint16_t lengthInBits[8],
-                                   const void * const job_in_lane[8]);
+                                   const void * const job_in_lane[8],
+                                   const uint64_t tag_size);
 
+IMB_DLL_LOCAL
+void zuc256_eia3_8_buffer_job_gfni_avx2(const void * const pKey[8],
+                                   const uint8_t *ivs,
+                                   const void * const pBufferIn[8],
+                                   void *pMacI[8],
+                                   const uint16_t lengthInBits[8],
+                                   const void * const job_in_lane[8],
+                                   const uint64_t tag_size);
 /* the s-boxes */
 extern const uint8_t S0[256];
 extern const uint8_t S1[256];
