@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2022, Intel Corporation
+  Copyright (c) 2022-2023, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -70,6 +70,7 @@
 #define SUBMIT_HASH_BURST submit_hash_burst_sse_t1
 #define SUBMIT_HASH_BURST_NOCHECK submit_hash_burst_nocheck_sse_t1
 
+#define SET_SUITE_ID_FN   set_suite_id_sse_t1
 
 /* Hash */
 #define SUBMIT_JOB_HASH    SUBMIT_JOB_HASH_SSE
@@ -366,6 +367,7 @@ init_mb_mgr_sse_t1_internal(IMB_MGR *state, const int reset_mgrs)
         state->submit_cipher_burst_nocheck = SUBMIT_CIPHER_BURST_NOCHECK;
         state->submit_hash_burst   = SUBMIT_HASH_BURST;
         state->submit_hash_burst_nocheck = SUBMIT_HASH_BURST_NOCHECK;
+        state->set_suite_id        = SET_SUITE_ID_FN;
 
         state->keyexp_128          = aes_keyexp_128_sse;
         state->keyexp_192          = aes_keyexp_192_sse;
@@ -390,6 +392,7 @@ init_mb_mgr_sse_t1_internal(IMB_MGR *state, const int reset_mgrs)
         state->md5_one_block       = md5_one_block_sse;
 
         state->aes128_cfb_one      = aes_cfb_128_one_sse;
+        state->aes256_cfb_one      = aes_cfb_256_one_sse;
 
         state->eea3_1_buffer       = zuc_eea3_1_buffer_sse;
         state->eea3_4_buffer       = zuc_eea3_4_buffer_sse;
@@ -485,6 +488,9 @@ init_mb_mgr_sse_t1_internal(IMB_MGR *state, const int reset_mgrs)
         state->gmac128_finalize    = imb_aes_gmac_finalize_128_sse;
         state->gmac192_finalize    = imb_aes_gmac_finalize_192_sse;
         state->gmac256_finalize    = imb_aes_gmac_finalize_256_sse;
+
+        state->aes_ecb_128_quic    = aes_ecb_quic_enc_128_sse;
+        state->aes_ecb_256_quic    = aes_ecb_quic_enc_256_sse;
 }
 
 #include "mb_mgr_code.h"
