@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2018-2021, Intel Corporation
+  Copyright (c) 2018-2023, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -29,6 +29,7 @@
 #include "intel-ipsec-mb.h"
 #include "gcm.h"
 #include "noaesni.h"
+#include "error.h"
 
 /**
  * @brief Pre-processes GCM key data
@@ -44,39 +45,63 @@
 void aes_gcm_pre_128_sse(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_128_enc_sse(key, key_data->expanded_keys);
         aes_gcm_precomp_128_sse(key_data);
 }
 
+#ifdef AESNI_EMU
 void aes_gcm_pre_128_sse_no_aesni(const void *key,
                                   struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_128_enc_sse_no_aesni(key, key_data->expanded_keys);
         aes_gcm_precomp_128_sse_no_aesni(key_data);
 }
+#endif /* AESNI_EMU */
 
 void aes_gcm_pre_128_avx_gen2(const void *key, struct gcm_key_data *key_data)
 {
-#ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
-                return;
-#endif
-        aes_keyexp_128_enc_avx(key, key_data->expanded_keys);
-        aes_gcm_precomp_128_avx_gen2(key_data);
+        aes_gcm_pre_128_sse(key, key_data);
 }
 
 void aes_gcm_pre_128_avx_gen4(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_128_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_128_avx_gen4(key_data);
@@ -85,8 +110,17 @@ void aes_gcm_pre_128_avx_gen4(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_128_avx512(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_128_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_128_avx512(key_data);
@@ -95,8 +129,17 @@ void aes_gcm_pre_128_avx512(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_128_vaes_avx512(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_128_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_128_vaes_avx512(key_data);
@@ -105,39 +148,63 @@ void aes_gcm_pre_128_vaes_avx512(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_192_sse(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_192_enc_sse(key, key_data->expanded_keys);
         aes_gcm_precomp_192_sse(key_data);
 }
 
+#ifdef AESNI_EMU
 void aes_gcm_pre_192_sse_no_aesni(const void *key,
                                   struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_192_enc_sse_no_aesni(key, key_data->expanded_keys);
         aes_gcm_precomp_192_sse_no_aesni(key_data);
 }
+#endif /* AESNI_EMU */
 
 void aes_gcm_pre_192_avx_gen2(const void *key, struct gcm_key_data *key_data)
 {
-#ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
-                return;
-#endif
-        aes_keyexp_192_enc_avx(key, key_data->expanded_keys);
-        aes_gcm_precomp_192_avx_gen2(key_data);
+        aes_gcm_pre_192_sse(key, key_data);
 }
 
 void aes_gcm_pre_192_avx_gen4(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_192_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_192_avx_gen4(key_data);
@@ -146,8 +213,17 @@ void aes_gcm_pre_192_avx_gen4(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_192_avx512(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_192_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_192_avx512(key_data);
@@ -156,8 +232,17 @@ void aes_gcm_pre_192_avx512(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_192_vaes_avx512(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_192_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_192_vaes_avx512(key_data);
@@ -166,39 +251,63 @@ void aes_gcm_pre_192_vaes_avx512(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_256_sse(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_256_enc_sse(key, key_data->expanded_keys);
         aes_gcm_precomp_256_sse(key_data);
 }
 
+#ifdef AESNI_EMU
 void aes_gcm_pre_256_sse_no_aesni(const void *key,
                                   struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_256_enc_sse_no_aesni(key, key_data->expanded_keys);
         aes_gcm_precomp_256_sse_no_aesni(key_data);
 }
+#endif /* AESNI_EMU */
 
 void aes_gcm_pre_256_avx_gen2(const void *key, struct gcm_key_data *key_data)
 {
-#ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
-                return;
-#endif
-        aes_keyexp_256_enc_avx(key, key_data->expanded_keys);
-        aes_gcm_precomp_256_avx_gen2(key_data);
+        aes_gcm_pre_256_sse(key, key_data);
 }
 
 void aes_gcm_pre_256_avx_gen4(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_256_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_256_avx_gen4(key_data);
@@ -207,8 +316,17 @@ void aes_gcm_pre_256_avx_gen4(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_256_avx512(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_256_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_256_avx512(key_data);
@@ -217,8 +335,17 @@ void aes_gcm_pre_256_avx512(const void *key, struct gcm_key_data *key_data)
 void aes_gcm_pre_256_vaes_avx512(const void *key, struct gcm_key_data *key_data)
 {
 #ifdef SAFE_PARAM
-        if (key == NULL || key_data == NULL)
+        /* reset error status */
+        imb_set_errno(NULL, 0);
+
+        if (key == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_KEY);
                 return;
+        }
+        if (key_data == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
+                return;
+        }
 #endif
         aes_keyexp_256_enc_avx2(key, key_data->expanded_keys);
         aes_gcm_precomp_256_vaes_avx512(key_data);
@@ -290,8 +417,8 @@ imb_aes_gmac_init_128_avx_gen2(const struct gcm_key_data *key_data,
                                const uint8_t *iv,
                                const uint64_t iv_len)
 {
-        aes_gcm_init_var_iv_128_avx_gen2(key_data, context_data, iv,
-                                         iv_len, NULL, 0);
+        aes_gcm_init_var_iv_128_sse(key_data, context_data, iv,
+                                    iv_len, NULL, 0);
 }
 
 void
@@ -300,8 +427,8 @@ imb_aes_gmac_init_192_avx_gen2(const struct gcm_key_data *key_data,
                                const uint8_t *iv,
                                const uint64_t iv_len)
 {
-        aes_gcm_init_var_iv_192_avx_gen2(key_data, context_data, iv,
-                                         iv_len, NULL, 0);
+        aes_gcm_init_var_iv_192_sse(key_data, context_data, iv,
+                                    iv_len, NULL, 0);
 }
 
 void
@@ -310,8 +437,8 @@ imb_aes_gmac_init_256_avx_gen2(const struct gcm_key_data *key_data,
                                const uint8_t *iv,
                                const uint64_t iv_len)
 {
-        aes_gcm_init_var_iv_256_avx_gen2(key_data, context_data, iv,
-                                         iv_len, NULL, 0);
+        aes_gcm_init_var_iv_256_sse(key_data, context_data, iv,
+                                    iv_len, NULL, 0);
 }
 
 void
@@ -320,8 +447,8 @@ imb_aes_gmac_finalize_128_avx_gen2(const struct gcm_key_data *key_data,
                                    uint8_t *auth_tag,
                                    const uint64_t  auth_tag_len)
 {
-        aes_gcm_enc_128_finalize_avx_gen2(key_data, context_data, auth_tag,
-                                          auth_tag_len);
+        aes_gcm_enc_128_finalize_sse(key_data, context_data, auth_tag,
+                                     auth_tag_len);
 }
 
 void
@@ -330,8 +457,8 @@ imb_aes_gmac_finalize_192_avx_gen2(const struct gcm_key_data *key_data,
                                    uint8_t *auth_tag,
                                    const uint64_t  auth_tag_len)
 {
-        aes_gcm_enc_192_finalize_avx_gen2(key_data, context_data, auth_tag,
-                                          auth_tag_len);
+        aes_gcm_enc_192_finalize_sse(key_data, context_data, auth_tag,
+                                     auth_tag_len);
 }
 
 void
@@ -340,10 +467,11 @@ imb_aes_gmac_finalize_256_avx_gen2(const struct gcm_key_data *key_data,
                                    uint8_t *auth_tag,
                                    const uint64_t  auth_tag_len)
 {
-        aes_gcm_enc_256_finalize_avx_gen2(key_data, context_data, auth_tag,
-                                          auth_tag_len);
+        aes_gcm_enc_256_finalize_sse(key_data, context_data, auth_tag,
+                                     auth_tag_len);
 }
 
+#ifdef AESNI_EMU
 void
 imb_aes_gmac_init_128_sse_no_aesni(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
@@ -403,6 +531,7 @@ imb_aes_gmac_finalize_256_sse_no_aesni(const struct gcm_key_data *key_data,
         aes_gcm_enc_256_finalize_sse_no_aesni(key_data, context_data, auth_tag,
                                               auth_tag_len);
 }
+#endif /* AESNI_EMU */
 
 void
 imb_aes_gmac_init_128_avx_gen4(const struct gcm_key_data *key_data,
