@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2022, Intel Corporation
+  Copyright (c) 2022-2023, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -32,6 +32,8 @@
 #include "intel-ipsec-mb.h"
 #include "arch_x86_64.h"
 
+#ifndef NO_SELF_TEST_DEV
+
 static int process_job(IMB_MGR *p_mgr)
 {
         IMB_JOB *job = IMB_SUBMIT_JOB(p_mgr);
@@ -45,7 +47,7 @@ static int process_job(IMB_MGR *p_mgr)
 
                 /* flush to get the job processed */
                 job = IMB_FLUSH_JOB(p_mgr);
-                
+
                 /* if flush returns nothing then it's an error */
                 if (!job)
                         return 0;
@@ -230,6 +232,111 @@ static const uint8_t aes_ctr_256_cipher_text[] = {
         0x56, 0x08, 0x63, 0xDC, 0x71, 0xE3, 0xE0, 0xC0,
 };
 
+/*
+ * AES-ECB test vectors
+ */
+
+static const uint8_t aes_ecb_128_key[] = {
+        0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6,
+        0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c
+};
+static const uint8_t aes_ecb_128_plain_text[] = {
+        0x6b, 0xc1, 0xbe, 0xe2, 0x2e, 0x40, 0x9f, 0x96,
+        0xe9, 0x3d, 0x7e, 0x11, 0x73, 0x93, 0x17, 0x2a,
+        0xae, 0x2d, 0x8a, 0x57, 0x1e, 0x03, 0xac, 0x9c,
+        0x9e, 0xb7, 0x6f, 0xac, 0x45, 0xaf, 0x8e, 0x51,
+        0x30, 0xc8, 0x1c, 0x46, 0xa3, 0x5c, 0xe4, 0x11,
+        0xe5, 0xfb, 0xc1, 0x19, 0x1a, 0x0a, 0x52, 0xef,
+        0xf6, 0x9f, 0x24, 0x45, 0xdf, 0x4f, 0x9b, 0x17,
+        0xad, 0x2b, 0x41, 0x7b, 0xe6, 0x6c, 0x37, 0x10
+};
+static const uint8_t aes_ecb_128_cipher_text[] = {
+        0x3a, 0xd7, 0x7b, 0xb4, 0x0d, 0x7a, 0x36, 0x60,
+        0xa8, 0x9e, 0xca, 0xf3, 0x24, 0x66, 0xef, 0x97,
+        0xf5, 0xd3, 0xd5, 0x85, 0x03, 0xb9, 0x69, 0x9d,
+        0xe7, 0x85, 0x89, 0x5a, 0x96, 0xfd, 0xba, 0xaf,
+        0x43, 0xb1, 0xcd, 0x7f, 0x59, 0x8e, 0xce, 0x23,
+        0x88, 0x1b, 0x00, 0xe3, 0xed, 0x03, 0x06, 0x88,
+        0x7b, 0x0c, 0x78, 0x5e, 0x27, 0xe8, 0xad, 0x3f,
+        0x82, 0x23, 0x20, 0x71, 0x04, 0x72, 0x5d, 0xd4
+};
+
+static const uint8_t aes_ecb_192_key[] = {
+        0x8e, 0x73, 0xb0, 0xf7, 0xda, 0x0e, 0x64, 0x52,
+        0xc8, 0x10, 0xf3, 0x2b, 0x80, 0x90, 0x79, 0xe5,
+        0x62, 0xf8, 0xea, 0xd2, 0x52, 0x2c, 0x6b, 0x7b
+};
+static const uint8_t aes_ecb_192_plain_text[] = {
+        0x6b, 0xc1, 0xbe, 0xe2, 0x2e, 0x40, 0x9f, 0x96,
+        0xe9, 0x3d, 0x7e, 0x11, 0x73, 0x93, 0x17, 0x2a,
+        0xae, 0x2d, 0x8a, 0x57, 0x1e, 0x03, 0xac, 0x9c,
+        0x9e, 0xb7, 0x6f, 0xac, 0x45, 0xaf, 0x8e, 0x51,
+        0x30, 0xc8, 0x1c, 0x46, 0xa3, 0x5c, 0xe4, 0x11,
+        0xe5, 0xfb, 0xc1, 0x19, 0x1a, 0x0a, 0x52, 0xef,
+        0xf6, 0x9f, 0x24, 0x45, 0xdf, 0x4f, 0x9b, 0x17,
+        0xad, 0x2b, 0x41, 0x7b, 0xe6, 0x6c, 0x37, 0x10
+};
+static const uint8_t aes_ecb_192_cipher_text[] = {
+        0xbd, 0x33, 0x4f, 0x1d, 0x6e, 0x45, 0xf2, 0x5f,
+        0xf7, 0x12, 0xa2, 0x14, 0x57, 0x1f, 0xa5, 0xcc,
+        0x97, 0x41, 0x04, 0x84, 0x6d, 0x0a, 0xd3, 0xad,
+        0x77, 0x34, 0xec, 0xb3, 0xec, 0xee, 0x4e, 0xef,
+        0xef, 0x7a, 0xfd, 0x22, 0x70, 0xe2, 0xe6, 0x0a,
+        0xdc, 0xe0, 0xba, 0x2f, 0xac, 0xe6, 0x44, 0x4e,
+        0x9a, 0x4b, 0x41, 0xba, 0x73, 0x8d, 0x6c, 0x72,
+        0xfb, 0x16, 0x69, 0x16, 0x03, 0xc1, 0x8e, 0x0e
+};
+
+static const uint8_t aes_ecb_256_key[] = {
+        0x60, 0x3d, 0xeb, 0x10, 0x15, 0xca, 0x71, 0xbe,
+        0x2b, 0x73, 0xae, 0xf0, 0x85, 0x7d, 0x77, 0x81,
+        0x1f, 0x35, 0x2c, 0x07, 0x3b, 0x61, 0x08, 0xd7,
+        0x2d, 0x98, 0x10, 0xa3, 0x09, 0x14, 0xdf, 0xf4
+};
+static const uint8_t aes_ecb_256_plain_text[] = {
+        0x6b, 0xc1, 0xbe, 0xe2, 0x2e, 0x40, 0x9f, 0x96,
+        0xe9, 0x3d, 0x7e, 0x11, 0x73, 0x93, 0x17, 0x2a,
+        0xae, 0x2d, 0x8a, 0x57, 0x1e, 0x03, 0xac, 0x9c,
+        0x9e, 0xb7, 0x6f, 0xac, 0x45, 0xaf, 0x8e, 0x51,
+        0x30, 0xc8, 0x1c, 0x46, 0xa3, 0x5c, 0xe4, 0x11,
+        0xe5, 0xfb, 0xc1, 0x19, 0x1a, 0x0a, 0x52, 0xef,
+        0xf6, 0x9f, 0x24, 0x45, 0xdf, 0x4f, 0x9b, 0x17,
+        0xad, 0x2b, 0x41, 0x7b, 0xe6, 0x6c, 0x37, 0x10
+};
+static const uint8_t aes_ecb_256_cipher_text[] = {
+        0xf3, 0xee, 0xd1, 0xbd, 0xb5, 0xd2, 0xa0, 0x3c,
+        0x06, 0x4b, 0x5a, 0x7e, 0x3d, 0xb1, 0x81, 0xf8,
+        0x59, 0x1c, 0xcb, 0x10, 0xd4, 0x10, 0xed, 0x26,
+        0xdc, 0x5b, 0xa7, 0x4a, 0x31, 0x36, 0x28, 0x70,
+        0xb6, 0xed, 0x21, 0xb9, 0x9c, 0xa6, 0xf4, 0xf9,
+        0xf1, 0x53, 0xe7, 0xb1, 0xbe, 0xaf, 0xed, 0x1d,
+        0x23, 0x30, 0x4b, 0x7a, 0x39, 0xf9, 0xf3, 0xff,
+        0x06, 0x7d, 0x8d, 0x8f, 0x9e, 0x24, 0xec, 0xc7
+};
+static const uint8_t null_iv[] = { 0x00 };
+
+/*
+ * Triple DES test vector TDES_EDE_CBC
+ */
+static const uint8_t tdes_ede_cbc_key[] = {
+        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
+        0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
+        0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17
+};
+
+static const uint8_t tdes_ede_cbc_iv[] = {
+        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07
+};
+
+static const uint8_t tdes_ede_cbc_plain_text[] = {
+        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
+        0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f
+};
+
+static const uint8_t tdes_ede_cbc_cipher_text[] = {
+        0x89, 0x4b, 0xc3, 0x08, 0x54, 0x26, 0xa4, 0x41,
+        0xf2, 0x7f, 0x73, 0xae, 0x26, 0xab, 0xbf, 0x74
+};
 
 #define ADD_CIPHER_VECTOR(_cmode,_key,_iv,_plain,_cipher) \
         {_cmode, _key, sizeof(_key), _iv, sizeof(_iv), \
@@ -248,13 +355,33 @@ struct self_test_cipher_vector cipher_vectors[] = {
                           aes_ctr_192_plain_text, aes_ctr_192_cipher_text),
         ADD_CIPHER_VECTOR(IMB_CIPHER_CNTR, aes_ctr_256_key, aes_ctr_256_iv,
                           aes_ctr_256_plain_text, aes_ctr_256_cipher_text),
+        ADD_CIPHER_VECTOR(IMB_CIPHER_ECB, aes_ecb_128_key, null_iv,
+                          aes_ecb_128_plain_text, aes_ecb_128_cipher_text),
+        ADD_CIPHER_VECTOR(IMB_CIPHER_ECB, aes_ecb_192_key, null_iv,
+                          aes_ecb_192_plain_text, aes_ecb_192_cipher_text),
+        ADD_CIPHER_VECTOR(IMB_CIPHER_ECB, aes_ecb_256_key, null_iv,
+                          aes_ecb_256_plain_text, aes_ecb_256_cipher_text),
+        ADD_CIPHER_VECTOR(IMB_CIPHER_DES3, tdes_ede_cbc_key, tdes_ede_cbc_iv,
+                          tdes_ede_cbc_plain_text, tdes_ede_cbc_cipher_text),
 };
 
+#define DES_KEY_SCHED_WORDS (IMB_DES_KEY_SCHED_SIZE / sizeof(uint64_t))
+
 static int self_test_ciphers(IMB_MGR *p_mgr)
 {
+        union {
+                struct {
+                        DECLARE_ALIGNED(uint32_t expkey_enc[4*15], 16);
+                        DECLARE_ALIGNED(uint32_t expkey_dec[4*15], 16);
+                } aes;
+                struct {
+                        uint64_t key_sched1[DES_KEY_SCHED_WORDS];
+                        uint64_t key_sched2[DES_KEY_SCHED_WORDS];
+                        uint64_t key_sched3[DES_KEY_SCHED_WORDS];
+                        void *keys[3];
+                } tdes;
+        } ks;
         uint8_t scratch[256];
-        DECLARE_ALIGNED(uint32_t expkey_enc[4*15], 16);
-        DECLARE_ALIGNED(uint32_t expkey_dec[4*15], 16);
         unsigned i;
 
         while (IMB_FLUSH_JOB(p_mgr) != NULL)
@@ -269,22 +396,39 @@ static int self_test_ciphers(IMB_MGR *p_mgr)
                 if (v->plain_text_size > sizeof(scratch))
                         return 0;
 
-                switch (v->cipher_key_size) {
-                case IMB_KEY_128_BYTES:
-                        IMB_AES_KEYEXP_128(p_mgr, v->cipher_key,
-                                           expkey_enc, expkey_dec);
-                        break;
-                case IMB_KEY_192_BYTES:
-                        IMB_AES_KEYEXP_192(p_mgr, v->cipher_key,
-                                           expkey_enc, expkey_dec);
-                        break;
-                case IMB_KEY_256_BYTES:
-                        IMB_AES_KEYEXP_256(p_mgr, v->cipher_key,
-                                           expkey_enc, expkey_dec);
-                        break;
-                default:
-                        /* invalid key size */
-                        return 0;
+                if (v->cipher_mode == IMB_CIPHER_DES3) {
+                        if (v->cipher_key_size != IMB_KEY_192_BYTES) {
+                                /* invalid key size */
+                                return 0;
+                        }
+                        des_key_schedule(ks.tdes.key_sched1, &v->cipher_key[0]);
+                        des_key_schedule(ks.tdes.key_sched2, &v->cipher_key[8]);
+                        des_key_schedule(ks.tdes.key_sched3,
+                                         &v->cipher_key[16]);
+                        ks.tdes.keys[0] = ks.tdes.key_sched1;
+                        ks.tdes.keys[1] = ks.tdes.key_sched2;
+                        ks.tdes.keys[2] = ks.tdes.key_sched3;
+                } else {
+                        switch (v->cipher_key_size) {
+                        case IMB_KEY_128_BYTES:
+                                IMB_AES_KEYEXP_128(p_mgr, v->cipher_key,
+                                                   ks.aes.expkey_enc,
+                                                   ks.aes.expkey_dec);
+                                break;
+                        case IMB_KEY_192_BYTES:
+                                IMB_AES_KEYEXP_192(p_mgr, v->cipher_key,
+                                                   ks.aes.expkey_enc,
+                                                   ks.aes.expkey_dec);
+                                break;
+                        case IMB_KEY_256_BYTES:
+                                IMB_AES_KEYEXP_256(p_mgr, v->cipher_key,
+                                                   ks.aes.expkey_enc,
+                                                   ks.aes.expkey_dec);
+                                break;
+                        default:
+                                /* invalid key size */
+                                return 0;
+                        }
                 }
 
                 /* test encrypt direction */
@@ -296,12 +440,18 @@ static int self_test_ciphers(IMB_MGR *p_mgr)
                 job->src = v->plain_text;
                 job->dst = scratch;
                 job->cipher_mode = v->cipher_mode;
-                job->enc_keys = expkey_enc;
-                if (v->cipher_mode != IMB_CIPHER_CNTR)
-                        job->dec_keys = expkey_dec;
+                if (v->cipher_mode == IMB_CIPHER_DES3) {
+                        job->enc_keys = ks.tdes.keys;
+                } else {
+                        job->enc_keys = ks.aes.expkey_enc;
+                        if (v->cipher_mode != IMB_CIPHER_CNTR)
+                                job->dec_keys = ks.aes.expkey_dec;
+                }
                 job->key_len_in_bytes = v->cipher_key_size;
-                job->iv = v->cipher_iv;
-                job->iv_len_in_bytes = v->cipher_iv_size;
+                if (v->cipher_mode != IMB_CIPHER_ECB) {
+                        job->iv = v->cipher_iv;
+                        job->iv_len_in_bytes = v->cipher_iv_size;
+                }
                 job->cipher_start_src_offset_in_bytes = 0;
                 job->msg_len_to_cipher_in_bytes = v->plain_text_size;
 
@@ -324,12 +474,18 @@ static int self_test_ciphers(IMB_MGR *p_mgr)
                 job->src = v->cipher_text;
                 job->dst = scratch;
                 job->cipher_mode = v->cipher_mode;
-                job->dec_keys = expkey_dec;
-                if (v->cipher_mode == IMB_CIPHER_CNTR)
-                        job->enc_keys = expkey_enc;
+                if (v->cipher_mode == IMB_CIPHER_DES3) {
+                        job->dec_keys = ks.tdes.keys;
+                } else {
+                        job->dec_keys = ks.aes.expkey_dec;
+                        if (v->cipher_mode == IMB_CIPHER_CNTR)
+                                job->enc_keys = ks.aes.expkey_enc;
+                }
                 job->key_len_in_bytes = v->cipher_key_size;
-                job->iv = v->cipher_iv;
-                job->iv_len_in_bytes = v->cipher_iv_size;
+                if (v->cipher_mode != IMB_CIPHER_ECB) {
+                        job->iv = v->cipher_iv;
+                        job->iv_len_in_bytes = v->cipher_iv_size;
+                }
                 job->cipher_start_src_offset_in_bytes = 0;
                 job->msg_len_to_cipher_in_bytes = v->plain_text_size;
 
@@ -342,7 +498,7 @@ static int self_test_ciphers(IMB_MGR *p_mgr)
                 /* check for plain text mismatch */
                 if (memcmp(scratch, v->plain_text, v->plain_text_size))
                         return 0;
-                
+
         } /* for(cipher_vectors) */
 
         return 1;
@@ -781,84 +937,18 @@ static int self_test_hash(IMB_MGR *p_mgr)
                 job->msg_len_to_hash_in_bytes = v->message_size;
                 job->auth_tag_output = scratch;
                 job->auth_tag_output_len_in_bytes = v->tag_size;
-
-                if (v->hash_mode == IMB_AUTH_HMAC_SHA_1) {
-                        /* compute IPAD and OPAD */
-                        unsigned j;
-
-                        IMB_ASSERT(sizeof(scratch) >= IMB_SHA1_BLOCK_SIZE);
-
-                        memset(scratch, 0x36, IMB_SHA1_BLOCK_SIZE);
-                        for (j = 0; j < v->hash_key_size; j++)
-                                scratch[j] ^= v->hash_key[j];
-                        IMB_SHA1_ONE_BLOCK(p_mgr, scratch, hmac_ipad);
-
-                        memset(scratch, 0x5c, IMB_SHA1_BLOCK_SIZE);
-                        for (j = 0; j < v->hash_key_size; j++)
-                                scratch[j] ^= v->hash_key[j];
-                        IMB_SHA1_ONE_BLOCK(p_mgr, scratch, hmac_opad);
-
+                if (v->hash_mode >= IMB_AUTH_HMAC_SHA_1 &&
+                    v->hash_mode <= IMB_AUTH_HMAC_SHA_512) {
+                        imb_hmac_ipad_opad(p_mgr, v->hash_mode, v->hash_key,
+                                           v->hash_key_size,
+                                           hmac_ipad, hmac_opad);
                         job->u.HMAC._hashed_auth_key_xor_ipad = hmac_ipad;
                         job->u.HMAC._hashed_auth_key_xor_opad = hmac_opad;
                 }
-
-                if (v->hash_mode == IMB_AUTH_HMAC_SHA_224 ||
-                    v->hash_mode == IMB_AUTH_HMAC_SHA_256) {
-                        /* compute IPAD and OPAD */
-                        unsigned j;
-
-                        IMB_ASSERT(sizeof(scratch) >= IMB_SHA_256_BLOCK_SIZE);
-
-                        memset(scratch, 0x36, IMB_SHA_256_BLOCK_SIZE);
-                        for (j = 0; j < v->hash_key_size; j++)
-                                scratch[j] ^= v->hash_key[j];
-                        if (v->hash_mode == IMB_AUTH_HMAC_SHA_224)
-                                IMB_SHA224_ONE_BLOCK(p_mgr, scratch, hmac_ipad);
-                        else
-                                IMB_SHA256_ONE_BLOCK(p_mgr, scratch, hmac_ipad);
-
-                        memset(scratch, 0x5c, IMB_SHA_256_BLOCK_SIZE);
-                        for (j = 0; j < v->hash_key_size; j++)
-                                scratch[j] ^= v->hash_key[j];
-                        if (v->hash_mode == IMB_AUTH_HMAC_SHA_224)
-                                IMB_SHA224_ONE_BLOCK(p_mgr, scratch, hmac_opad);
-                        else
-                                IMB_SHA256_ONE_BLOCK(p_mgr, scratch, hmac_opad);
-
-                        job->u.HMAC._hashed_auth_key_xor_ipad = hmac_ipad;
-                        job->u.HMAC._hashed_auth_key_xor_opad = hmac_opad;
-                }
-
-                if (v->hash_mode == IMB_AUTH_HMAC_SHA_384 ||
-                    v->hash_mode == IMB_AUTH_HMAC_SHA_512) {
-                        /* compute IPAD and OPAD */
-                        unsigned j;
-
-                        IMB_ASSERT(sizeof(scratch) >= IMB_SHA_512_BLOCK_SIZE);
-
-                        memset(scratch, 0x36, IMB_SHA_512_BLOCK_SIZE);
-                        for (j = 0; j < v->hash_key_size; j++)
-                                scratch[j] ^= v->hash_key[j];
-                        if (v->hash_mode == IMB_AUTH_HMAC_SHA_384)
-                                IMB_SHA384_ONE_BLOCK(p_mgr, scratch, hmac_ipad);
-                        else
-                                IMB_SHA512_ONE_BLOCK(p_mgr, scratch, hmac_ipad);
-
-                        memset(scratch, 0x5c, IMB_SHA_512_BLOCK_SIZE);
-                        for (j = 0; j < v->hash_key_size; j++)
-                                scratch[j] ^= v->hash_key[j];
-                        if (v->hash_mode == IMB_AUTH_HMAC_SHA_384)
-                                IMB_SHA384_ONE_BLOCK(p_mgr, scratch, hmac_opad);
-                        else
-                                IMB_SHA512_ONE_BLOCK(p_mgr, scratch, hmac_opad);
-
-                        job->u.HMAC._hashed_auth_key_xor_ipad = hmac_ipad;
-                        job->u.HMAC._hashed_auth_key_xor_opad = hmac_opad;
-                }
-
                 if (v->hash_mode == IMB_AUTH_AES_CMAC) {
                         IMB_AES_KEYEXP_128(p_mgr, v->hash_key, expkey, dust);
-                        IMB_AES_CMAC_SUBKEY_GEN_128(p_mgr, expkey, skey1, skey2);
+                        IMB_AES_CMAC_SUBKEY_GEN_128(p_mgr, expkey,
+                                                    skey1, skey2);
                         job->u.CMAC._key_expanded = expkey;
                         job->u.CMAC._skey1 = skey1;
                         job->u.CMAC._skey2 = skey2;
@@ -866,7 +956,8 @@ static int self_test_hash(IMB_MGR *p_mgr)
 
                 if (v->hash_mode == IMB_AUTH_AES_CMAC_256) {
                         IMB_AES_KEYEXP_256(p_mgr, v->hash_key, expkey, dust);
-                        IMB_AES_CMAC_SUBKEY_GEN_256(p_mgr, expkey, skey1, skey2);
+                        IMB_AES_CMAC_SUBKEY_GEN_256(p_mgr, expkey,
+                                                    skey1, skey2);
                         job->u.CMAC._key_expanded = expkey;
                         job->u.CMAC._skey1 = skey1;
                         job->u.CMAC._skey2 = skey2;
@@ -878,7 +969,7 @@ static int self_test_hash(IMB_MGR *p_mgr)
                         job->u.GMAC._iv = v->hash_iv;
                         job->u.GMAC.iv_len_in_bytes = v->hash_iv_size;
                 }
-                
+
                 if (v->hash_mode == IMB_AUTH_AES_GMAC_192) {
                         IMB_AES192_GCM_PRE(p_mgr, v->hash_key, &gmac_key);
                         job->u.GMAC._key = &gmac_key;
@@ -1563,6 +1654,8 @@ static int self_test_aead(IMB_MGR *p_mgr)
         return 1;
 }
 
+#endif /* NO_SELF_TEST_DEV */
+
 /*
  * =============================================================================
  * SELF-TEST INTERNAL API
@@ -1573,6 +1666,10 @@ IMB_DLL_LOCAL int self_test(IMB_MGR *p_mgr)
 {
         int ret = 1;
 
+#ifdef NO_SELF_TEST_DEV
+        p_mgr->features &= ~(IMB_FEATURE_SELF_TEST |
+                             IMB_FEATURE_SELF_TEST_PASS);
+#else
         p_mgr->features |= IMB_FEATURE_SELF_TEST;
         p_mgr->features &= ~IMB_FEATURE_SELF_TEST_PASS;
 
@@ -1588,10 +1685,6 @@ IMB_DLL_LOCAL int self_test(IMB_MGR *p_mgr)
         if (ret)
                 p_mgr->features |= IMB_FEATURE_SELF_TEST_PASS;
 
-#ifdef NO_SELF_TEST_DEV
-        p_mgr->features &= ~(IMB_FEATURE_SELF_TEST |
-                             IMB_FEATURE_SELF_TEST_PASS);
-        ret = 1;
 #endif
 
         return ret;
